@@ -6,7 +6,7 @@ use crate::clocks::*;
 use crate::constants::*;
 use crate::storage::*;
 
-/// A *clock constraint* bounding the difference of two clocks.
+/// A *clock constraint* bounding the difference between two clocks.
 pub struct Constraint<B: Bound> {
     pub(crate) left: Clock,
     pub(crate) right: Clock,
@@ -151,6 +151,12 @@ pub trait Zone<B: Bound> {
     /// Returns the number of clock variables of this zone.
     fn num_variables(&self) -> usize;
 
+    /// Returns the number of clocks of this zone.
+    ///
+    /// Note: This is always `num_variables + 1` as there is the constant
+    /// zero clock plus the clock variables.
+    fn num_clocks(&self) -> usize;
+
     /// Retrieves the difference bound for `left - right`.
     fn get_bound(&self, left: impl AnyClock, right: impl AnyClock) -> &B;
 
@@ -199,6 +205,18 @@ pub trait Zone<B: Bound> {
     fn resize(&self, num_variables: usize) -> Self;
 }
 
+/// Returns an iterator over the clocks of a zone.
+#[inline(always)]
+pub fn clocks<Z: Zone<B>, B: Bound>(zone: &Z) -> impl Iterator<Item = Clock> {
+    (0..zone.num_clocks()).map(|index| Clock(index))
+}
+
+/// Returns an iterator over the variables of a zone.
+#[inline(always)]
+pub fn variables<Z: Zone<B>, B: Bound>(zone: &Z) -> impl Iterator<Item = Variable> {
+    (1..zone.num_clocks()).map(|index| Variable(index))
+}
+
 /// An implementation of [Zone] as *difference bound matrix*.
 ///
 /// Uses [LinearLayout] as the default [storage layout][Layout].
@@ -299,6 +317,11 @@ impl<B: Bound, L: Layout<B>> Zone<B> for Dbm<B, L> {
         self.dimension - 1
     }
 
+    #[inline(always)]
+    fn num_clocks(&self) -> usize {
+        self.dimension
+    }
+
     #[inline(always)]
     fn get_bound(&self, left: impl AnyClock, right: impl AnyClock) -> &B {
         self.layout.get(left, right)
